/*
 * PGA450Comm.c
 *
 *  Created on: Aug 10, 2012
 *      Author: a0135618
 */


#include <msp430.h>
#include "main.h"
#if defined(__MSP430G2231__)
#include "USI_SPI.h"
#elif defined(__MSP430G2553__)
#include "USCI_SPI.h"
#endif
#include "PGA450_MP_Setup.h"
#include "PGA450Comm.h"
#include "PGA450FW.h"
#include "PGA450_EEPROM.h"


#define PGA450_CS_PORT      P2OUT
#define PGA450_CS_PIN       0x01

#define PGA450_LED_PORT2    P2OUT
#define PGA450_LED_DIR2     P2DIR
#define PGA450_LED_MASK2    0x22

#define PGA450_LED_PORT1    P1OUT
#define PGA450_LED_DIR1     P1DIR
#define PGA450_LED_MASK1    0x1A

#define PGA450_VPWR_PORT    P1OUT
#define PGA450_VPWR_PIN     0x01
#define PGA450_VOTP_PORT    P2OUT
#define PGA450_VOTP_PIN     0x80

#define pga450VPWROn()      (PGA450_VPWR_PORT &= ~PGA450_VPWR_PIN)
#define pga450VPWROff()     (PGA450_VPWR_PORT |= PGA450_VPWR_PIN)
#define pga450VOTPOn()      (PGA450_VOTP_PORT &= ~PGA450_VOTP_PIN)
#define pga450VOTPOff()     (PGA450_VOTP_PORT |= PGA450_VOTP_PIN)

#define pga450TaskDly       timer16[PGA450_TASK_DLY]

unsigned char pga450id;
unsigned char pga450Cnt;
unsigned char pga450Stg;
unsigned char pga450Buf[4];


volatile unsigned char* const PGA450CSPort[] = {&P2OUT, &P2OUT, &P2OUT, &P2OUT, &P1OUT};
unsigned char const PGA450CSMask[] = {0x01, 0x04, 0x10, 0x40, 0x04};


void pga450Disable(unsigned char id)
{
	unsigned int i;

	if(id==6)
		{
			for(i=0; i<5; i++)
			{
				*PGA450CSPort[i] |= PGA450CSMask[i];
			}
		}
	else
	{
		*PGA450CSPort[id] |= PGA450CSMask[id];
	}
}


void pga450Enable(unsigned char id)
{
	unsigned int i;

	if (id == 6)
	{
		for(i=0; i<5; i++)
		{
			*PGA450CSPort[i] &= ~PGA450CSMask[i];
		}
	}
	else
	{
		*PGA450CSPort[id] &= ~PGA450CSMask[id];
	}
}


volatile unsigned char* const PGA450LEDPort[] = {&P2OUT, &P1OUT, &P1OUT, &P1OUT, &P2OUT};
volatile unsigned char* const PGA450LEDDir[] = {&P2DIR, &P1DIR, &P1DIR, &P1DIR, &P2DIR};
unsigned char const PGA450LEDMask[] = {0x02, 0x10, 0x08, 0x02, 0x20};


void pga450LedOK(unsigned char id)
{
    *PGA450LEDPort[id] |= PGA450LEDMask[id];
    *PGA450LEDDir[id] |= PGA450LEDMask[id];
}


void pga450LedErr(unsigned char id)
{
    *PGA450LEDPort[id] &= ~PGA450LEDMask[id];
    *PGA450LEDDir[id] |= PGA450LEDMask[id];
}


void pga450LedIdle(unsigned char id)
{
    *PGA450LEDPort[id] &= ~PGA450LEDMask[id];
    *PGA450LEDDir[id] &= ~PGA450LEDMask[id];
}


void pga450LedBusy(unsigned char id)
{
    *PGA450LEDPort[id] ^= PGA450LEDMask[id];
    *PGA450LEDDir[id] |= PGA450LEDMask[id];
}


void pga450LedReset(void)
{
    PGA450_LED_PORT1 &= ~PGA450_LED_MASK1;
    PGA450_LED_DIR1 &= ~PGA450_LED_MASK1;

    PGA450_LED_PORT2 &= ~PGA450_LED_MASK2;
    PGA450_LED_DIR2 &= ~PGA450_LED_MASK2;
}


// Write data to PGA450
unsigned char pga450Write(unsigned char id, unsigned char length)	//id = 6 = all, OTP write
{
    unsigned char status;
    unsigned char *pBuf;
    unsigned char all;

    pBuf = pga450Buf;
    if(id==6)
    {
    	all = 1;
    }
    else
    {
    	all=0;
    }

    pga450Enable(id);

    #if defined(__MSP430G2231__)
      status = usiSPIWriteRead(*pBuf++);
    #elif defined(__MSP430G2553__)
      status = ucsiSPIWriteRead(*pBuf++, all);
    #endif

    while(--length)
    {
        #if defined(__MSP430G2231__)
          (void)usiSPIWriteRead(*pBuf++);
        #elif defined(__MSP430G2553__)
          (void)ucsiSPIWriteRead(*pBuf++, all);
        #endif
    }

    pga450Disable(id);

    //pga450LedBusy(id);

    return status;
}


// Read data from PGA450
// Data is available on next read call
unsigned char pga450Read(unsigned char id, unsigned char length)
{
    unsigned char status;
    unsigned char *pBuf;

    pBuf = pga450Buf;

    pga450Enable(id);

    #if defined(__MSP430G2231__)
        status = usiSPIWriteRead(*pBuf++);
    #elif defined(__MSP430G2553__)
        status = ucsiSPIWriteRead(*pBuf++, 0);
    #endif


    while(--length)
    {
        #if defined(__MSP430G2231__)
            *pBuf = usiSPIWriteRead(*pBuf);
        #elif defined(__MSP430G2553__)
            *pBuf = ucsiSPIWriteRead(*pBuf, 0);
        #endif
        pBuf++;
    }

    pga450Disable(id);

    //pga450LedBusy(id);

    return status;
}


// Reset PGA450
unsigned char pga450Reset(unsigned char id)
{
    unsigned int i;
    unsigned char retry;

    freezeDog();

    retry = PGA450_CMD_RETRY_TIME;
    for(i=0; i<2; i++) // Write 2 times to confirm the command has been sent successfully
    {
        pga450Buf[0] = PGA450_TEST_REG_WRITE_CMD;
        pga450Buf[1] = PGA450_MICRO_RESET_REG_ADDR;
        pga450Buf[2] = PGA450_MICRO_RESET_CMD;
        if(pga450Write(id, 3) != PGA450_OK)
        {
            i = 0;
            retry--;
            if(retry == 0)
                return 1;
        }
    }

    for(i=0; i<2; i++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        for(;;)
        {
            pga450Buf[0] = PGA450_TEST_REG_READ_CMD;
            pga450Buf[1] = PGA450_MICRO_RESET_REG_ADDR;
            pga450Buf[2] = PGA450_MICRO_RESET_CMD;
            if(pga450Read(id, 3) != PGA450_OK)
            {
                retry--;
                if(retry == 0)
                    return 1;
            }
            else
            {
                if(i > 0)
                {
                    if(pga450Buf[1] != PGA450_MICRO_RESET_CMD)
                    {
                        retry--;
                        if(retry == 0)
                            return 1;
                    }
                    else
                        break;
                }
                else
                    break;
            }
        }
    }

    return 0;
}


// PGA450 OTP Blank Check
unsigned char pga450OTP_BlankChk(unsigned char id)
{
    unsigned int i, k;
    unsigned char retry, status;

    freezeDog();

    for(i=PGA450_OTP_ADDR_START; i<=(PGA450_OTP_ADDR_END+1); i++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        k = 0;
        for(;;)
        {
            pga450Buf[0] = PGA450_OTP_READ_CMD;
            pga450Buf[1] = i>>8;
            pga450Buf[2] = i;
            pga450Buf[3] = 0;

            status = pga450Read(id, 4);

            if(k == 1) // Unwanted read if error, skip
            {
                k++;
            }
            else
            {
                if((i > PGA450_OTP_ADDR_START) || (k > 0))
                {
                    if((status != PGA450_OK) || (pga450Buf[1] != 0))
                    {
                        if(k == 0)
                        {
                            k++;
                            i--;
                        }
                        else
                        {
                            retry--;
                            if(retry == 0)
                                return 2;
                        }
                    }
                    else
                        break;
                }
                else
                    break;
            }
        }
    }

    return 0;
}


// PGA450 OTP Programming
unsigned char pga450OTP_Program(unsigned char id)
{
    unsigned int i, j;
    unsigned char retry;

    freezeDog();

    for(i=PGA450_OTP_ADDR_START, j=0; ((i<=PGA450_OTP_ADDR_END) && (j<PGA450_FW_SZ)); i++, j++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        for(;;)
        {
            pga450Buf[3] = pga450FW[j];
            #if (PGA450_DEBUG == 1)
              pga450Buf[0] = PGA450_DEVRAM_WRITE_CMD;
            #else
              pga450Buf[0] = PGA450_OTP_WRITE_CMD;
            #endif
            pga450Buf[1] = i>>8;
            pga450Buf[2] = i;

            if(pga450Write(6, 4) != PGA450_OK)	//6 refers to writing to all PGA450s at once
            {
                retry--;
                if(retry == 0)
                    return 1;
                delay_us(160);
            }
            else
            {
                delay_us(160);
                break;
            }
        }
    }

    return 0;
}


// PGA450 OTP Verifying
unsigned char pga450OTP_Verify(unsigned char id)
{
    unsigned int i, j, k;
    unsigned char retry, status;

    freezeDog();

    for(i=PGA450_OTP_ADDR_START, j=0; ((i<=PGA450_OTP_ADDR_END) && (j<PGA450_FW_SZ)); i++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        k = 0;
        for(;;)
        {
            #if (PGA450_DEBUG == 1)
              pga450Buf[0] = PGA450_DEVRAM_READ_CMD;
            #else
              pga450Buf[0] = PGA450_OTP_READ_CMD;
            #endif
            pga450Buf[1] = i>>8;
            pga450Buf[2] = i;
            pga450Buf[3] = 0;

            status = pga450Read(id, 4);

            if(k == 1) // Unwanted read if error, skip
            {
                k++;
            }
            else
            {
                if((i > (j + PGA450_OTP_ADDR_START)) || (k > 0)) // First read is dummy, skip
                {
                    if((status != PGA450_OK) || (pga450Buf[1] != pga450FW[j]))
                    {
                        if(k == 0)
                        {
                            k++;
                            i--;
                        }
                        else
                        {
                            retry--;
                            if(retry == 0)
                                return 2;
                        }
                    }
                    else
                    {
                        j++;
                        break;
                    }
                }
                else
                    break;
            }
        }
    }

    return 0;
}


// PGA450 EEPROM Programming
unsigned char pga450EE_Program(unsigned char id)
{
    unsigned int i, j;
    unsigned char retry;

    freezeDog();

    // Write to cache
    for(i=PGA450_EE_ADDR_START, j=0; ((i<=PGA450_EE_ADDR_END) && (j<PGA450_EE_SZ)); i++, j++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        for(;;)
        {
            pga450Buf[0] = PGA450_EEPROM_CACHE_WRITE_CMD;
            pga450Buf[1] = i;
            pga450Buf[2] = pga450EE[j];

            if(pga450Write(6, 3) != PGA450_OK)	//6 refers to writing to all PGA450s at once
            {
                retry--;
                if(retry == 0)
                    return 1;
                //delay_us(50);
            }
            else
            {
                //delay_us(50);
                break;
            }
        }
    }

    // Write to non-volatile cells
    retry = PGA450_CMD_RETRY_TIME;
    for(i=0; i<2; i++) // Write 2 times to confirm the command has been sent successfully
    {
        pga450Buf[0] = PGA450_ESFR_WRITE_CMD;
        pga450Buf[1] = PGA450_EE_CTRL_REG_ADDR;
        pga450Buf[2] = PGA450_EE_WRTIE_BIT;
        if(pga450Write(id, 3) != PGA450_OK)
        {
            i = 0;
            retry--;
            if(retry == 0)
                return 1;
        }
    }

    delay_us(10000);

    for(i=0; i<2; i++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        for(;;)
        {
            pga450Buf[0] = PGA450_ESFR_READ_CMD;
            pga450Buf[1] = PGA450_EE_CTRL_REG_ADDR;
            pga450Buf[2] = 0;
            if(pga450Read(id, 3) != PGA450_OK)
            {
                retry--;
                if(retry == 0)
                    return 1;
            }
            else
            {
                if(i > 0)
                {
                    if(pga450Buf[1] & PGA450_EE_STATUS_BIT)
                    {
                        delay_us(10000);
                    }
                    else
                        break;
                }
                else
                    break;
            }
        }
    }

    return 0;
}


// PGA450 EEPROM Verifying
unsigned char pga450EE_Verify(unsigned char id)
{
    unsigned int i, j, k;
    unsigned char retry, status;

    freezeDog();

    // Reload from non-volatile cells to cache
    retry = PGA450_CMD_RETRY_TIME;
    for(i=0; i<2; i++) // Write 2 times to confirm the command has been sent successfully
    {
        pga450Buf[0] = PGA450_ESFR_WRITE_CMD;
        pga450Buf[1] = PGA450_EE_CTRL_REG_ADDR;
        pga450Buf[2] = PGA450_EE_RELOAD_BIT;
        if(pga450Write(id, 3) != PGA450_OK)
        {
            i = 0;
            retry--;
            if(retry == 0)
                return 1;
        }
    }

    delay_us(1000);

    // Read from cache
    for(i=PGA450_EE_ADDR_START, j=0; ((i<=PGA450_EE_ADDR_END) && (j<PGA450_EE_SZ)); i++)
    {
        retry = PGA450_CMD_RETRY_TIME;
        k = 0;
        for(;;)
        {
            pga450Buf[0] = PGA450_EEPROM_CACHE_READ_CMD;
            pga450Buf[1] = i;
            pga450Buf[2] = 0;

            status = pga450Read(id, 3);

            if(k == 1) // Unwanted read if error, skip
            {
                k++;
            }
            else
            {
                if((i > (j + PGA450_EE_ADDR_START)) || (k > 0))
                {
                    if((status != PGA450_OK) || (pga450Buf[1] != pga450EE[j]))
                    {
                        if(k == 0)
                        {
                            k++;
                            i--;
                        }
                        else
                        {
                            retry--;
                            if(retry == 0)
                                return 2;
                        }
                    }
                    else
                    {
                        j++;
                        break;
                    }
                }
                else
                    break;
            }
        }
    }

    return 0;
}


// PGA450 variables initialization
void pga450Init(void)
{
    pga450Stg = PGA450_STG_IDLE;
}


// PGA450 OTP programming task
void pga450ProgTask(void)
{
    switch(pga450Stg)
    {
        case PGA450_STG_INIT:
        {
            pga450LedReset();
            pga450Cnt = PGA450_RESET_RETRY_TIME;
            pga450VPWROn();
            pga450TaskDly = PGA450_POWER_ON_DLY;
            pga450Stg = PGA450_STG_VPWR_ON_DLY;
            break;
        }

        case PGA450_STG_VPWR_ON_DLY:
        {
            if(pga450TaskDly == 0)
            {
                pga450VOTPOn();
                pga450TaskDly = PGA450_VOTP_ON_DLY;
                pga450Stg = PGA450_STG_VOTP_ON_DLY;
            }
            break;
        }

        case PGA450_STG_VOTP_ON_DLY:
        {
            if(pga450TaskDly == 0)
            {
                pga450Stg = PGA450_STG_RESET;
            }
            break;
        }

        case PGA450_STG_RESET:
        {
        	for(pga450id=0; pga450id<PGA450_DEV_MAX; pga450id++)
        	{
        		pga450Reset(pga450id);

        	}
        	pga450Cnt = PGA450_BLANK_CHK_RETRY_TIME;
        	#if (PGA450_DEBUG == 1)
        		pga450Stg = PGA450_STG_PROG_OTP;
        	#else
        		pga450Cnt = PGA450_OTP_PROG_RETRY_TIME;
        		pga450Stg = PGA450_STG_PROG_OTP;
        	#endif

            break;
        }

        case PGA450_STG_BLANK_CHK:	//skip blank check since this error will be caught on verify step
        {
        	for(pga450id=0; pga450id<PGA450_DEV_MAX; pga450id++)
        	{
				if(pga450OTP_BlankChk(pga450id))
				{
					pga450Cnt--;
					if(pga450Cnt == 0)
					{
						pga450LedErr(pga450id);
						//pga450Stg = PGA450_STG_NEXT_DEV;
					}
				}
        	}
        	pga450Cnt = PGA450_OTP_PROG_RETRY_TIME;
            pga450Stg = PGA450_STG_PROG_OTP;

            break;
        }

        case PGA450_STG_PROG_OTP:
        {

        	pga450id = 6; //write to all pga450s at once
        		if(pga450OTP_Program(pga450id)==0)
        		{

				  #if (PGA450_EE_PROG_EN == 1)
					pga450Cnt = PGA450_EE_PROG_RETRY_TIME;
					pga450Stg = PGA450_STG_PROG_EE;
				  #else
					pga450Cnt = PGA450_OTP_VERIFY_RETRY_TIME;
					pga450Stg = PGA450_STG_VERIFY_OTP_EE;
				  #endif
        		}

            break;
        }

        case PGA450_STG_VERIFY_OTP_EE:
        {
        	for(pga450id=0; pga450id<PGA450_DEV_MAX; pga450id++)
        	{
				if(pga450OTP_Verify(pga450id) == 0)
				{

				  #if (PGA450_EE_PROG_EN == 0)
					pga450LedOK(pga450id);
				  #else
					if (pga450EE_Verify(pga450id)==0)
					{
						pga450LedOK(pga450id);
					}
				  #endif
				}
        	}

					pga450Stg = PGA450_STG_VOTP_OFF;

            break;
        }

        case PGA450_STG_PROG_EE:
        {
        	pga450id = 6; //write to all PGA450s at once

        	pga450EE_Program(pga450id);

			pga450Cnt = PGA450_OTP_VERIFY_RETRY_TIME;
			pga450Stg = PGA450_STG_VERIFY_OTP_EE;

            break;
        }

        case PGA450_STG_VOTP_OFF:
        {

                pga450VOTPOff();
                pga450TaskDly = PGA450_VOTP_OFF_DLY;
                pga450Stg = PGA450_STG_VOTP_OFF_DLY;

                break;
        }

        case PGA450_STG_VOTP_OFF_DLY:
        {
            if(pga450TaskDly == 0)
            {
                pga450VPWROff();
                pga450Stg = PGA450_STG_IDLE;
            }
            break;
        }

        default:
        {
            pga450VOTPOff();
            pga450VPWROff();
            pga450Stg = PGA450_STG_IDLE;
        }
    }
}

